@@ -108,11 +108,12 @@ public:
     Result<std::string> get_julia(const std::string &code,
         LocationManager &lm, diag::Diagnostics &diagnostics);
     Result<std::unique_ptr<MLIRModule>> get_mlir(
-        ASR::TranslationUnit_t &asr, diag::Diagnostics &diagnostics);
+        ASR::asr_t &asr, diag::Diagnostics &diagnostics);
     Result<std::string> get_fortran(const std::string &code,
         LocationManager &lm, diag::Diagnostics &diagnostics);
     Result<std::string> get_fmt(const std::string &code, LocationManager &lm,
         diag::Diagnostics &diagnostics);
+    Allocator &get_al() { return al; };
 
 private:
     Allocator al;
