@@ -68,7 +68,7 @@ bool lex_int(const unsigned char *s, const unsigned char *e, uint64_t &u,
 void lex_int_large(Allocator &al, const unsigned char *s,
     const unsigned char *e, BigInt::BigInt &u, Str &suffix);
 void lex_format(unsigned char *&cur, Location &loc,
-        unsigned char *&start);
+        unsigned char *&start, diag::Diagnostics &diagnostics, bool continue_compilation);
 
 
 } // namespace LCompilers::LFortran
