@@ -159,7 +159,7 @@ public:
     }
 
 
-    std::string get_type(const ASR::ttype_t *t) {
+    std::string get_type(const ASR::ttype_t *t, ASR::symbol_t *type_decl = nullptr) {
         std::string r = "";
         switch (t->type) {
             case ASR::ttypeType::Integer: {
@@ -249,6 +249,11 @@ public:
             } case ASR::ttypeType::CPtr: {
                 r = "type(c_ptr)";
                 break;
+            } case ASR::ttypeType::FunctionType: {
+                r = "procedure(";
+                r += ASRUtils::symbol_name(type_decl);
+                r += ")";
+                break;
             }
             default:
                 throw LCompilersException("The type `"
@@ -686,7 +691,7 @@ public:
     void visit_Variable(const ASR::Variable_t &x) {
         std::string r = indent;
         std::string dims = "(";
-        r += get_type(x.m_type);
+        r += get_type(x.m_type, x.m_type_declaration);
         switch (x.m_intent) {
             case ASR::intentType::In : {
                 r += ", intent(in)";
@@ -1404,6 +1409,8 @@ public:
         else if(intrinsic_func_name == "MoveAlloc") intrinsic_func_name = "move_alloc";
         else if(intrinsic_func_name == "CompilerOptions") intrinsic_func_name = "compiler_options";
         else if(intrinsic_func_name == "CompilerVersion") intrinsic_func_name = "compiler_version";
+        else if(intrinsic_func_name == "CommandArgumentCount") intrinsic_func_name = "command_argument_count";
+        else if(intrinsic_func_name == "ErfcScaled") intrinsic_func_name = "erfc_scaled";
         visit_IntrinsicElementalFunction_helper(out, intrinsic_func_name, x);
     }
 
@@ -1419,6 +1426,7 @@ public:
         std::string intrinsic_func_name = ASRUtils::get_intrinsic_name(static_cast<int64_t>(x.m_inquiry_id));
         if(intrinsic_func_name == "BitSize") intrinsic_func_name = "bit_size";
         else if(intrinsic_func_name == "NewLine") intrinsic_func_name = "new_line";
+        else if(intrinsic_func_name == "StorageSize") intrinsic_func_name = "storage_size";
         visit_TypeInquiry_helper(out, intrinsic_func_name, x);
     }
 
@@ -1642,6 +1650,20 @@ public:
         src = r;
     }
 
+    void visit_StringSection(const ASR::StringSection_t &x) {
+        std::string r = "";
+        visit_expr(*x.m_arg);
+        r += src;
+        r += "(";
+        visit_expr(*x.m_start);
+        r += src;
+        r += ":";
+        visit_expr(*x.m_end);
+        r += src;
+        r += ")";
+        src = r;
+    }
+
     void visit_StringConstant(const ASR::StringConstant_t &x) {
         src = "\"";
         if(std::strcmp(x.m_s, "\n") == 0) {
@@ -1895,6 +1917,7 @@ public:
             {ASR::cast_kindType::ComplexToReal, {"real", {4, 8}}},
             {ASR::cast_kindType::RealToComplex, {"cmplx", {4, 8}}},
             {ASR::cast_kindType::LogicalToInteger, {"int", {1, 2, 4, 8}}},
+            {ASR::cast_kindType::ComplexToInteger, {"int", {4, 8}}},
         };
 
         if (cast_map.find(x.m_kind) != cast_map.end()) {
@@ -1959,6 +1982,35 @@ public:
         src = "iachar(" + src + ")";
     }
 
+    void visit_ArrayIsContiguous(const ASR::ArrayIsContiguous_t &x) {
+        visit_expr(*x.m_array);
+        src = "is_contiguous(" + src + ")";
+    }
+
+    void visit_ReAlloc(const ASR::ReAlloc_t &x) {
+        std::string r = indent;
+        r += "reallocate(";
+        for (size_t i = 0; i < x.n_args; i++) {
+            visit_expr(*x.m_args[i].m_a);
+            r += src;
+            if (x.m_args[i].n_dims > 0) {
+                r += "(";
+                for (size_t j = 0; j < x.m_args[i].n_dims; j++) {
+                    visit_expr(*x.m_args[i].m_dims[j].m_length);
+                    r += src;
+                    if (j < x.m_args[i].n_dims - 1) r += ", ";
+                }
+                r += ")";
+            }
+            if (i < x.n_args - 1) r += ", ";
+        }
+        r += ")";
+        handle_line_truncation(r, 2);
+        r += "\n";
+        src = r;
+    }
+
+
     // void visit_SizeOfType(const ASR::SizeOfType_t &x) {}
 
     // void visit_PointerNullConstant(const ASR::PointerNullConstant_t &x) {}
