@@ -1,10 +1,10 @@
 passes = [
-        "simplifier",
+        "array_struct_temporary",
         "replace_arr_slice",
         "replace_openmp",
         "replace_function_call_in_declaration",
+        "replace_array_passed_in_function_call",
         "replace_array_op",
-        "replace_array_op_simplifier",
         "replace_class_constructor",
         "dead_code_removal",
         "replace_div_to_mul",
@@ -32,15 +32,14 @@ passes = [
         "replace_select_case",
         "replace_sign_from_value",
         "create_subroutine_from_function",
-        "create_subroutine_from_function_simplifier",
         "transform_optional_argument_functions",
         "unused_functions",
         "update_array_dim_intrinsic_calls",
         "replace_where",
-        "replace_where_simplifier",
         "unique_symbols",
         "insert_deallocate",
-        "promote_allocatable_to_nonallocatable"
+        "promote_allocatable_to_nonallocatable",
+        "replace_with_compile_time_values"
 ]
 
 
