@@ -118,7 +118,6 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(SubstrIndex)
         INTRINSIC_NAME_CASE(Range)
         INTRINSIC_NAME_CASE(Radix)
-        INTRINSIC_NAME_CASE(IsContiguous)
         INTRINSIC_NAME_CASE(StorageSize)
         INTRINSIC_NAME_CASE(Hypot)
         INTRINSIC_NAME_CASE(SelectedIntKind)
@@ -450,8 +449,6 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Sign::instantiate_Sign, &Sign::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Radix),
             {nullptr, &Radix::verify_args}},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::IsContiguous),
-            {nullptr, &IsContiguous::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::StorageSize),
             {nullptr, &StorageSize::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Scale),
@@ -819,8 +816,6 @@ namespace IntrinsicElementalFunctionRegistry {
             "ishftc"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Radix),
             "radix"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::IsContiguous),
-            "is_contiguous"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::StorageSize),
             "storage_size"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Scale),
@@ -1058,7 +1053,6 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"min", {&Min::create_Min, &Min::eval_Min}},
                 {"ishftc", {&Ishftc::create_Ishftc, &Ishftc::eval_Ishftc}},
                 {"radix", {&Radix::create_Radix, &Radix::eval_Radix}},
-                {"is_contiguous", {&IsContiguous::create_IsContiguous, &IsContiguous::eval_IsContiguous}},
                 {"out_of_range", {&OutOfRange::create_OutOfRange, &OutOfRange::eval_OutOfRange}},
                 {"storage_size", {&StorageSize::create_StorageSize, &StorageSize::eval_StorageSize}},
                 {"scale", {&Scale::create_Scale, &Scale::eval_Scale}},
